//////////////////////////////////////////////////////////////////////
// mobile.cpp: implementation for the Mobile class (Hierarchical Transformation).
// -Kyoung Shin Park (2014 Spring)
//////////////////////////////////////////////////////////////////////
#include "simpleMobile.h"
#include "color.h"

namespace moglclass
{
	
SimpleMobile::SimpleMobile()
{
	angle = 0.0f; // 0~360
	active = true;
	init();
}

SimpleMobile::~SimpleMobile()
{
	for (int i=0; i<8; i++) delete mobile[i];
	//for (int i=0; i<7; i++) delete material[i];
	std::cout << "~SimpleMobile" << std::endl;
}

void SimpleMobile::init()
{
	mobile[0] = new Parallelepiped(glm::vec3(0.0f, 5.0f, 0.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 3.0f, 0.0f));
	mobile[1] = new Parallelepiped(glm::vec3(-4.0f, 0.0f, 4.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(4.0f, 5.0f, -4.0f));
	mobile[2] = new Parallelepiped(glm::vec3(4.0f, 0.0f, 4.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(-4.0f, 5.0f, -4.0f));
	mobile[3] = new Parallelepiped(glm::vec3(0.0f, 0.0f, -4.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 5.0f, 4.0f));

	mobile[4] = new Cube(glm::vec3(0, 0, 0), 7.0f);
	mobile[5] = new Torus(glm::vec3(0, 0, 0), 3.0f, 1.0f, 32, 16);
	mobile[6] = new Sphere(glm::vec3(0, 0, 0), 3.5f, 16, 16);
	mobile[7] = new Cylinder(glm::vec3(0, 0, 0), 3.5f, 4.0f, 16);

	//material[0] = new Material(glm::vec4(0.1, 0.1, 0.1, 1.0), glm::vec4(0,1,0,1), glm::vec4(1.0, 1.0, 1.0, 1.0), 50);
	//material[1] = new Material(glm::vec4(0.1, 0.1, 0.1, 1.0), glm::vec4(0,0,1,1), glm::vec4(1.0, 1.0, 1.0, 1.0), 50);
	//material[2] = new Material(glm::vec4(0.1, 0.1, 0.1, 1.0), glm::vec4(1,0,0,1), glm::vec4(1.0, 1.0, 1.0, 1.0), 50);
	//material[3] = new Material(glm::vec4(0.1, 0.1, 0.1, 1.0), glm::vec4(1,1,0,1), glm::vec4(1.0, 1.0, 1.0, 1.0), 50);
	//material[4] = new Material(glm::vec4(0.1, 0.1, 0.1, 1.0), glm::vec4(0,1,1,1), glm::vec4(1.0, 1.0, 1.0, 1.0), 50);
	//material[5] = new Material(glm::vec4(0.1, 0.1, 0.1, 1.0), glm::vec4(1,0,1,1), glm::vec4(1.0, 1.0, 1.0, 1.0), 50);
	//material[6] = new Material(glm::vec4(0.1, 0.1, 0.1, 1.0), glm::vec4(0.7,0.5,1,1), glm::vec4(1.0, 1.0, 1.0, 1.0), 50);
}

void SimpleMobile::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
{
	if (p == NULL) return;

	p->useProgram();
	p->setUniform("gProjection", projection);
	p->setUniform("gView", view);

	glm::mat4 mainMatrix = model * bodyTransform;
	p->setUniform("gModel", mainMatrix);
	
	//material[0]->setUniform(p, "gMaterial");
	p->setUniform("gMaterialDiffuseColor", Color::Green); 
	if (mobile[0]) mobile[0]->draw();
	if (mobile[1]) mobile[1]->draw();
	//material[1]->setUniform(p, "gMaterial");
	p->setUniform("gMaterialDiffuseColor", Color::Blue); 
	if (mobile[2]) mobile[2]->draw();
	//material[2]->setUniform(p, "gMaterial");
	p->setUniform("gMaterialDiffuseColor", Color::Red); 
	if (mobile[3]) mobile[3]->draw();

	glm::mat4 mobileMatrix = model * bodyTransform * wheelTransform[0];
	p->setUniform("gModel", mobileMatrix);
	//material[3]->setUniform(p, "gMaterial");
	p->setUniform("gMaterialDiffuseColor", Color::Yellow); 
	if (mobile[4]) mobile[4]->draw();

	mobileMatrix = model * bodyTransform * wheelTransform[1];
	p->setUniform("gModel", mobileMatrix);
	//material[4]->setUniform(p, "gMaterial");
	p->setUniform("gMaterialDiffuseColor", glm::vec4(0,1,1,1)); 
	if (mobile[5]) mobile[5]->draw();

	mobileMatrix = model * bodyTransform * wheelTransform[2];
	p->setUniform("gModel", mobileMatrix);
	//material[5]->setUniform(p, "gMaterial");
	p->setUniform("gMaterialDiffuseColor", glm::vec4(1,0,1,1)); 
	if (mobile[6]) mobile[6]->draw();
	mobileMatrix = model * bodyTransform * wheelTransform[2] * glm::translate(glm::mat4(1.0f), glm::vec3(2, 2, 0));
	p->setUniform("gModel", mobileMatrix);
	if (mobile[6]) mobile[6]->draw();
	mobileMatrix = model * bodyTransform * wheelTransform[2] * glm::translate(glm::mat4(1.0f), glm::vec3(-2, 2, 0));
	p->setUniform("gModel", mobileMatrix);
	if (mobile[6]) mobile[6]->draw();

	mobileMatrix = model * bodyTransform * wheelTransform[2] * wheelTransform[3];
	p->setUniform("gModel", mobileMatrix);
	//material[6]->setUniform(p, "gMaterial");
	p->setUniform("gMaterialDiffuseColor", glm::vec4(0.7,0.5,1,1)); 
	if (mobile[7]) mobile[7]->draw();
}

bool SimpleMobile::update(float deltaTime)
{
	if (active)
	{
		angle = angle - 180.0f * (float) (deltaTime) * 0.000001f;
	}

	bodyTransform = glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0, 1, 0));
	wheelTransform[0] = glm::translate(glm::mat4(1.0f), glm::vec3(-4.0f, -3.0f, 4.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 0.0f, 1.0f)); // cube
	wheelTransform[1] = glm::translate(glm::mat4(1.0f), glm::vec3(4.0f, -3.0f, 4.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 1.0f, 0.0f)); // torus
	wheelTransform[2] = glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, -3.0f, -4.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 1.0f, 0.0f)); // sphere
	wheelTransform[3] = glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, -5.0f, 0.0f)) * glm::rotate(glm::mat4(1.0f), angle, glm::vec3(0.0f, 1.0f, 0.0f)); // cylinder

	return true;
}

} // moglclass namespace